#!/usr/bin/python
""" see bigquant doc
    DataSource("basic_info_IndustrySw").read(instruments=stock_lists)
    out: DataFrame
"""

from http import client
from rrdata.common import save_df_to_pgsql
from rrdata.common.engine_pgsql import engine
from sqlalchemy import table


class RrdataDSave(object):
    """ see:
        bigquant DataSource("trade_days").read() 
        also add  can change:
        save data(dataframe) to database-table by sql_driver
    """
    def __init__(self, table_name, 
               database=engine(driver="", db_name="rrdata"),
                ):
        """ driver: ""-default - poasgresql/ psycopg2   db_name  rrdata/rrshare/rrfactor"""
        self.table_name = table_name
        self.engine = database

    def __repr__(self) -> str:  # name 
        return f"<{self.__class__.__name__} table_name={self.table_name}>"

    #def read(self):  # read artubuite
    #    return  read_df_from_table(self.table_name, self.engine)

    def save(self, data):
        #print(f"----save data to {self.table_name}")
        return save_df_to_pgsql(data, self.table_name)


if  __name__ == "__main__":
    #from rrdata.rrdatad.stock.fetch_stock_list import  fetch_stock_list_tusharepro
    #data = fetch_stock_list_tusharepro()
    #table_name = 'stock_list_tspro'
    #print(RrdataDSave(table_name).__repr__())
    #RrdataDSave(table_name).save(data)
    from rrdata.rrdatad.stock.stock_zh_a_hist_em import stock_zh_a_spot_em
    RrdataDSave("stock_spot_20220516").save(stock_zh_a_spot_em())
    






# coding:utf-8
import asyncio
import asyncpg
import psycopg2

from rrdata.utils.config_setting import setting


host = setting['IP_DATABASE_ALIYUN']
db_name = setting["DATABASE"]
passwd = setting["PGSQL_PASSWORD"]
uri = f'postgresql://postgres:{passwd}@{host}:5432/{db_name}'

# connect client
def rq_util_sql_postgres_setting(uri=uri):
    # 使用uri代替ip,port的连接方式
    # 这样可以对postgresql进行加密:
    # uri=postgresql://user:passwor@ip:port/daatbase
    client = psycopg2.connect(uri)
    #print(client)
    return client


# async
def rq_util_sql_async_postgres_setting(uri=uri):  #TODO
    #loop = asyncio.new_event_loop()
    #asyncio.set_event_loop(loop)
    try:
        loop = asyncio.get_event_loop()
    except RuntimeError:
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
    # async def client():
    return loop
    #return loop.run_until_complete(uri)
    # yield  client()

##ASCENDING = psycopg2.ASCENDING
##DESCENDING = psycopg2.DESCENDING
#rq_util_sql_postgres_sort_ASCENDING = psycopg2.ASCENDING
#rq_util_sql_postgres_sort_DESCENDING = psycopg2.DESCENDING

if __name__ == '__main__':
    # test client
    print(rq_util_sql_postgres_setting())
    # test async_postgres
    client = rq_util_sql_async_postgres_setting()
    #client.rrdata.swl_list
    #print(client)




import psycopg2
import asyncio
import asyncpg
from sqlalchemy import create_engine, MetaData
from sqlalchemy.ext.asyncio import create_async_engine

from rrdata.utils.rqLogs import rq_util_log_info
from rrdata.utils.rqSetting import setting

host = setting["IP_DATABASE_ALIYUN"]
passwd = setting["PGSQL_PASSWORD"]
database_str = f"postgres:{passwd}@{host}:5432"
database_dns = f"postgresql://postgres:{passwd}@{host}:5432"
rq_util_log_info(database_dns)


def engine(driver="psycopg2", db_name="rrdata"):
    database_uri_str = f"{database_str}/{db_name}"
    if not driver:
        database_uri = f"postgresql://{database_uri_str}"
        #rq_util_log_info(database_uri)
        conn = create_engine(database_uri)
    elif driver == "psycopg2":
        conn = create_engine(f"postgresql+{driver}://{database_str}/{db_name}")
    return conn
    
async def engine_asyncpg(db_name='rrdata"'):
    """
           usage:
        conn = await asyncpg.connect()
        con = engine_asyncpg(db_name)
        asyncio.get_event_loop().run_until_complete(func())
    """
    return await asyncpg.connect(f"postgresql://{database_str}/{db_name}")

def engine_async(db_name="rrdata"): #TODO
    return create_async_engine(f"postgresql+asyncpg://{database_str}/{db_name}",
             #  connect_args={"server_settings": {"jit": "off"}},
             )


if  __name__ == "__main__":
    rq_util_log_info(engine(db_name='rrdata'))
    rq_util_log_info(engine_async())
    #rq_util_log_info(engine_asyncpg())
    
        


from rrdata.utils.rqLogs import rq_util_log_expection, rq_util_log_info
from rrdata.utils.rqPgsql import read_data_from_pg
from rrdata.utils.rqParameter import SWL_LEVEL

rq_util_log_info(SWL_LEVEL().LEVEL)

def get_index_sw_class(level=""):
    df = read_data_from_pg("swl_list",)
    #rq_util_log_info(f"swl index class from rrdara:swl_list. \n {df}")
    if not level:
        return df
    elif level in SWL_LEVEL().LEVEL:
        df = df[df.level == level]
        return df
    else:
        rq_util_log_expection("swl level code is not exist ! ")


if  __name__ == '__main__':
    rq_util_log_info(f"swl index class from rrdara:swl_list. \n {get_index_sw_class()}")
    for l in SWL_LEVEL().LEVEL:
        rq_util_log_info(f"swl level: {l}\n{get_index_sw_class(l)}")

    
    





from sqlalchemy import create_engine
import pandas as pd

import rrdata.rrdatad.index as index
from rrdata.utils.rqLogs import rq_util_log_info, rq_util_log_expection
from rrdata.utils.rqSetting import setting
from rrdata.common import engine


def save_df_to_pgsql(data, table_name, db_name='rrdata', if_exists='replace', index=False):
    rq_util_log_info(data)
    try:
        data.to_sql(table_name, con=engine(db_name=db_name), if_exists=if_exists, index=False)
        rq_util_log_info(f"Saved {len(data)} rows to Table:<{table_name}>/DB:<{db_name}>, finish !!!")
    except Exception  as e:
        rq_util_log_expection(e)
        

if  __name__ == '__main__':
    #data = index.sw_index_class_all()
    #save_df_to_pgsql(data,'swl_list')
    #data2 = index.sw_index_cons()
    #save_df_to_pgsql(data2, 'swl_cons')
    data3 = index.sw_index_spot(level="L2")
    save_df_to_pgsql(data3, 'swl_spot_L2',db_name='rrshare')








#!/usr/bin/python
""" see bigquant doc
    DataSource('表名').read(instruments,start_date='2005-01-01', end_date=None, fields=['字段1','字段2'...])
    DataSource("basic_info_IndustrySw").read(instruments=stock_lists)
    参数 :
    表名  不同的表名存放了不同字段的历史数据，见 数据表名
    instruments  字符串列表，股票代码列表，见 股票列表
    start_date  字符串，开始日期
    end_date  字符串，结束日期
    fields  str 字符串数组，请求的字段列表，详见 数据字段
    返回:
    历史数据
    DataFrame
"""
from dataclasses import fields
from re import I
import pandas as pd
from typing import List, TypeVar
from typing_extensions import Self
from pandas import DataFrame
from rrdata.common import read_df_from_table, save_df_to_pgsql
from rrdata.common.engine_pgsql import engine


class RrdataD(object):
    """ see:
        bigquant DataSource("trade_days").read() 
        also add  can change:
        source database/sql_driver
    """
    def __init__(self,
                table_name, 
                datasource=engine(driver="", db_name="rrdata"),
                ):
        """ driver: ""-default - poasgresql/ psycopg2   db_name  rrdata/rrshare/rrfactor"""
        self.table_name = table_name
        self.engine = datasource

    def __repr__(self) -> str:  # name 
        return f"<{self.__class__.__name__} table_name={self.table_name}>"

    def read(self, instruments: List=None,start_date: str=None, end_date: str=None,fields: List=None) -> DataFrame:
        """   default None (all)  --> out whole table """

        '''
        if (instruments) & (not start_date ) & (not fields) :
         
            sql_query = f"""SELECT *  FROM {self.table_name}
                          WHERE ts_code IN {instruments} ;
                         """   
        elif (not instruments) & start_date & (not fields):
            sql_query = f"""SELECT *  FROM {self.table_name}
                            WHERE trade_date  >= '{start_date}';
                         """   
        elif (not instruments) & start_date & end_date & (not fields):
            sql_query = f"""SELECT *  FROM {self.table_name}
                            WHERE trade_date BETWEEN '{start_date}' AND '{end_date}';
                         """       
        elif fields:
             sql_query = f"""SELECT {fields}  FROM {self.table_name}
                            WHERE trade_date BETWEEN '{start_date}' AND '{end_date}' ;
        
                         """   
        '''
        
        if fields:
            field =  ','.join(fields) if isinstance(fields, list)  else fields 
        else:
            field ="*"
        sql_query = f"""SELECT DISTINCT {field}  
                        FROM {self.table_name}
                        """
        if start_date:
            sql_query += f"WHERE trade_date >= '{start_date}' "
        if end_date:
            sql_query += f"AND trade_date <= '{end_date}' "
        if instruments:
            if isinstance(instruments, list):
                instrument = "','".join(instruments)
            else:
                instrument = "','".join(instruments.split(",")) #map(lambda x: ",".join('x'), instruments)
                #print(instrument)
            ITERMS = "WHERE" if not start_date else "AND" 
            sql_query +=  f"{ITERMS} ts_code in ('{instrument}')"
        print(sql_query)
        return  pd.read_sql(sql_query, self.engine)


if  __name__ == "__main__":
    print(RrdataD("swl_list").__repr__())
    #print(RrdataD('stock_list').read())
    #print(RrdataD('stock_list_tspro').read(fields=["ts_code", "name", "list_status"]))
    #print(RrdataD('stock_spot').read(fields="code,pb, pe"))
    #print(RrdataD('stock_day_test').read(start_date='2022-05-16', end_date='2022-05-16', fields="ts_code, trade_date, close"))
    #print(RrdataD('stock_day_test').read(instruments=["000792.SZ", "600519.SH"]))
    #print(RrdataD('stock_day_test').read(instruments="000792.SZ,600519.SH"))

    print(RrdataD('stock_day_test').read(start_date='2022-05-16'))



import logging
import pstats
from symtable import Symbol
import pandas as pd

from rrdata.utils.rqCode import rq_util_str_tounitecode
from rrdata.utils.rqLogs import rq_util_log_debug, rq_util_log_info, rq_util_log_expection
from rrdata.rrdatad.stock import stock_zh_a_hist_em, stock_zh_a_spot_em
from rrdata.common import save_df_to_pgsql, read_df_from_table


startDate = "20220501"
def get_stock_daily_one(symbol="600519",period="daily",start_date=startDate, end_date="20321231", adjust="hfq"):
    """ 
    hist daily from eastmoney
    adjust: : "hfq"
    #HowTO get stock adhust_factor
    """
    df = stock_zh_a_hist_em(symbol,period,start_date,end_date,adjust)
    df['ts_code'] = rq_util_str_tounitecode(symbol)
    df = df[['ts_code','trade_date','open','high','low','close','chg_pct','vol','amount']]
    return df

def save_stock_daily(stock_list=None,db_name='stock_day'):
    """TODO  use asyncio -- postgres+asyncpg://host/dbname --engine
        5000 class to 20 class 250 unit
    """
    lists = read_df_from_table('stock_list').symbol.values if None else stock_list
    rq_util_log_debug(lists)
    for symbol in lists:
        try:
            data = get_stock_daily_one(symbol)
            save_df_to_pgsql(data,db_name,if_exists='append')
        except Exception as e:
            rq_util_log_expection(e)
            
   

if  __name__ == "__main__":
    list1 = ['600519','000792','835185']
    save_stock_daily(list1, 'stock_day_test')
   
    read_df_from_table('stock_day_test')
   





#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2022/2/2 23:26
Desc: 东方财富网-行情首页-沪深京 A 股 
    spot / hist days/ min 
    hist daily default : ajust:"qfq"

columns_new1 = [
            "no",
            "code",
            "name",
            "close",
            "chg_pct",
            "change",
            "vol",
            "amount",
            "amplitude",
            "high",
            "low",
            "open",
            "pre_close",
            "vol_chg",
            "turnover",
            "pe",
            "pb",
        ]
"""
import requests
import pandas as pd
from sqlalchemy import column



def stock_zh_a_spot_em() -> pd.DataFrame:
    """
    东方财富网-沪深京 A 股-实时行情
    http://quote.eastmoney.com/center/gridlist.html#hs_a_board
    :return: 实时行情
    :rtype: pandas.DataFrame
    """
    url = "http://82.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "5000",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f3",
        "fs": "m:0 t:6,m:0 t:80,m:1 t:2,m:1 t:23,m:0 t:81 s:2048",
        "fields": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,f18,f20,f21,f23,f24,f25,f22,f11,f62,f128,f136,f115,f152",
        "_": "1623833739532",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    if not data_json["data"]["diff"]:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json["data"]["diff"])
    temp_df.columns = [
        "_",
        "最新价",
        "涨跌幅",
        "涨跌额",
        "成交量",
        "成交额",
        "振幅",
        "换手率",
        "市盈率-动态",
        "量比",
        "_",
        "代码",
        "_",
        "名称",
        "最高",
        "最低",
        "今开",
        "昨收",
        "_",
        "_",
        "_",
        "市净率",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df.index + 1
    temp_df.rename(columns={"index": "序号"}, inplace=True)
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌幅",
            "涨跌额",
            "成交量",
            "成交额",
            "振幅",
            "最高",
            "最低",
            "今开",
            "昨收",
            "量比",
            "换手率",
            "市盈率-动态",
            "市净率",
        ]
    ]
    temp_df.columns = [
            "no",
            "code",
            "name",
            "close",
            "chg_pct",
            "change",
            "vol",
            "amount",
            "amplitude",
            "high",
            "low",
            "open",
            "pre_close",
            "vol_chg",
            "turnover",
            "pe",
            "pb",
        ]
    
    temp_df["close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    temp_df["chg_pct"] = pd.to_numeric(temp_df["chg_pct"], errors="coerce")
    temp_df["change"] = pd.to_numeric(temp_df["change"], errors="coerce")
    temp_df["vol"] = pd.to_numeric(temp_df["vol"], errors="coerce")
    temp_df["amount"] = pd.to_numeric(temp_df["amount"], errors="coerce")
    temp_df["amplitude"] = pd.to_numeric(temp_df["amplitude"], errors="coerce")
    temp_df["high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df["low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df["open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df["pre_close"] = pd.to_numeric(temp_df["pre_close"], errors="coerce")
    temp_df["vol_chg"] = pd.to_numeric(temp_df["vol_chg"], errors="coerce")
    temp_df["turnover"] = pd.to_numeric(temp_df["turnover"], errors="coerce")
    temp_df["pe"] = pd.to_numeric(temp_df["pe"], errors="coerce")
    temp_df["pb"] = pd.to_numeric(temp_df["pb"], errors="coerce")
    return temp_df


def code_id_map_em() -> dict:
    """
    东方财富-股票和市场代码
    http://quote.eastmoney.com/center/gridlist.html#hs_a_board
    :return: 股票和市场代码
    :rtype: dict
    """
    url = "http://80.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "5000",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f3",
        "fs": "m:1 t:2,m:1 t:23",
        "fields": "f12",
        "_": "1623833739532",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    if not data_json["data"]["diff"]:
        return dict()
    temp_df = pd.DataFrame(data_json["data"]["diff"])
    temp_df["market_id"] = 1
    temp_df.columns = ["sh_code", "sh_id"]
    code_id_dict = dict(zip(temp_df["sh_code"], temp_df["sh_id"]))
    params = {
        "pn": "1",
        "pz": "5000",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f3",
        "fs": "m:0 t:6,m:0 t:80",
        "fields": "f12",
        "_": "1623833739532",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    if not data_json["data"]["diff"]:
        return dict()
    temp_df_sz = pd.DataFrame(data_json["data"]["diff"])
    temp_df_sz["sz_id"] = 0
    code_id_dict.update(dict(zip(temp_df_sz["f12"], temp_df_sz["sz_id"])))
    params = {
        "pn": "1",
        "pz": "5000",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f3",
        "fs": "m:0 t:81 s:2048",
        "fields": "f12",
        "_": "1623833739532",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    if not data_json["data"]["diff"]:
        return dict()
    temp_df_sz = pd.DataFrame(data_json["data"]["diff"])
    temp_df_sz["bj_id"] = 0
    code_id_dict.update(dict(zip(temp_df_sz["f12"], temp_df_sz["bj_id"])))
    return code_id_dict


def stock_zh_a_hist_em(
        symbol: str = "000016",
        period: str = "daily",
        start_date: str = "19700101",
        end_date: str = "22220101",
        adjust: str = "qfq",
) -> pd.DataFrame:
    """
    东方财富网-行情首页-沪深京 A 股-每日行情
    http://quote.eastmoney.com/concept/sh603777.html?from=classic
    :param symbol: 股票代码
    :type symbol: str
    :param period: choice of {'daily', 'weekly', 'monthly'}
    :type period: str
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :param adjust: choice of {"qfq": "前复权", "hfq": "后复权", "": "不复权"}
    :type adjust: str
    :return: 每日行情
    :rtype: pandas.DataFrame
    """
    code_id_dict = code_id_map_em()
    adjust_dict = {"qfq": "1", "hfq": "2", "": "0"}
    period_dict = {'daily': '101', 'weekly': '102', 'monthly': '103'}
    url = "http://push2his.eastmoney.com/api/qt/stock/kline/get"
    params = {
        "fields1": "f1,f2,f3,f4,f5,f6",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f116",
        "ut": "7eea3edcaed734bea9cbfc24409ed989",
        "klt": period_dict[period],
        "fqt": adjust_dict[adjust],
        "secid": f"{code_id_dict[symbol]}.{symbol}",
        "beg": "0",
        "end": "20500000",
        "_": "1623766962675",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]["klines"]])
    temp_df.columns = [
        "日期",
        "开盘",
        "收盘",
        "最高",
        "最低",
        "成交量",
        "成交额",
        "振幅",
        "涨跌幅",
        "涨跌额",
        "换手率",
    ]
    temp_df.index = pd.to_datetime(temp_df["日期"])
    temp_df = temp_df[start_date:end_date]
    temp_df.reset_index(inplace=True, drop=True)
    
    temp_df['开盘'] = pd.to_numeric(temp_df['开盘'])
    temp_df['收盘'] = pd.to_numeric(temp_df['收盘'])
    temp_df['最高'] = pd.to_numeric(temp_df['最高'])
    temp_df['最低'] = pd.to_numeric(temp_df['最低'])
    temp_df['成交量'] = pd.to_numeric(temp_df['成交量'])
    temp_df['成交额'] = pd.to_numeric(temp_df['成交额'])
    temp_df['振幅'] = pd.to_numeric(temp_df['振幅'])
    temp_df['涨跌幅'] = pd.to_numeric(temp_df['涨跌幅'])
    temp_df['涨跌额'] = pd.to_numeric(temp_df['涨跌额'])
    temp_df['换手率'] = pd.to_numeric(temp_df['换手率'])

    temp_df.columns = [
        "trade_date",
        "open",
        "close",
        "high",
        "low",
        "vol",
        "amount",
        "amplitude",
        "chg_pct",
        "change",
        "turnover"
    ]
    return temp_df


def stock_zh_a_hist_min_em(
        symbol: str = "000001",
        start_date: str = "1979-09-01 09:32:00",
        end_date: str = "2222-01-01 09:32:00",
        period: str = '5',
        adjust: str = '',
) -> pd.DataFrame:
    """
    东方财富网-行情首页-沪深京 A 股-每日分时行情
    http://quote.eastmoney.com/concept/sh603777.html?from=classic
    :param symbol: 股票代码
    :type symbol: str
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :param period: choice of {'1', '5', '15', '30', '60'}
    :type period: str
    :param adjust: choice of {'', 'qfq', 'hfq'}
    :type adjust: str
    :return: 每日分时行情
    :rtype: pandas.DataFrame
    """
    code_id_dict = code_id_map_em()
    adjust_map = {
        '': '0',
        'qfq': '1',
        'hfq': '2',
    }
    if period == '1':
        url = 'https://push2his.eastmoney.com/api/qt/stock/trends2/get'
        params = {
            "fields1": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13",
            "fields2": "f51,f52,f53,f54,f55,f56,f57,f58",
            "ut": "7eea3edcaed734bea9cbfc24409ed989",
            "ndays": "5",
            "iscr": "0",
            "secid": f"{code_id_dict[symbol]}.{symbol}",
            "_": "1623766962675",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]["trends"]])
        temp_df.columns = [
            "时间",
            "开盘",
            "收盘",
            "最高",
            "最低",
            "成交量",
            "成交额",
            "最新价",
        ]
        temp_df.index = pd.to_datetime(temp_df["时间"])
        temp_df = temp_df[start_date:end_date]
        temp_df.reset_index(drop=True, inplace=True)
        temp_df["开盘"] = pd.to_numeric(temp_df["开盘"])
        temp_df["收盘"] = pd.to_numeric(temp_df["收盘"])
        temp_df["最高"] = pd.to_numeric(temp_df["最高"])
        temp_df["最低"] = pd.to_numeric(temp_df["最低"])
        temp_df["成交量"] = pd.to_numeric(temp_df["成交量"])
        temp_df["成交额"] = pd.to_numeric(temp_df["成交额"])
        temp_df["最新价"] = pd.to_numeric(temp_df["最新价"])
        temp_df['时间'] = pd.to_datetime(temp_df['时间']).astype(str)

        temp_df.columns = [
        "time",
        "open",
        "close",
        "high",
        "low",
        "vol",
        "amount",
        "last"
        ]
        return temp_df
    else:
        url = 'http://push2his.eastmoney.com/api/qt/stock/kline/get'
        params = {
            'fields1': 'f1,f2,f3,f4,f5,f6',
            'fields2': 'f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61',
            'ut': '7eea3edcaed734bea9cbfc24409ed989',
            'klt': period,
            'fqt': adjust_map[adjust],
            'secid': f"{code_id_dict[symbol]}.{symbol}",
            'beg': '0',
            'end': '20500000',
            '_': '1630930917857',
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]["klines"]])
        temp_df.columns = [
            "时间",
            "开盘",
            "收盘",
            "最高",
            "最低",
            "成交量",
            "成交额",
            "振幅",
            "涨跌幅",
            "涨跌额",
            "换手率",
        ]
        temp_df.index = pd.to_datetime(temp_df["时间"])
        temp_df = temp_df[start_date:end_date]
        temp_df.reset_index(drop=True, inplace=True)
        temp_df["开盘"] = pd.to_numeric(temp_df["开盘"])
        temp_df["收盘"] = pd.to_numeric(temp_df["收盘"])
        temp_df["最高"] = pd.to_numeric(temp_df["最高"])
        temp_df["最低"] = pd.to_numeric(temp_df["最低"])
        temp_df["成交量"] = pd.to_numeric(temp_df["成交量"])
        temp_df["成交额"] = pd.to_numeric(temp_df["成交额"])
        temp_df["振幅"] = pd.to_numeric(temp_df["振幅"])
        temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"])
        temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"])
        temp_df["换手率"] = pd.to_numeric(temp_df["换手率"])
        temp_df['时间'] = pd.to_datetime(temp_df['时间']).astype(str)
        temp_df = temp_df[[
            "时间",
            "开盘",
            "收盘",
            "最高",
            "最低",
            "涨跌幅",
            "涨跌额",
            "成交量",
            "成交额",
            "振幅",
            "换手率",
        ]]
        temp_df.columns = [
        "time",
        "open",
        "close",
        "high",
        "low",
        "chg_pct",
        "change",
        "vol",
        "amount",
        "amplitude",
        "turnover",
        ]
        return temp_df


def stock_zh_a_hist_pre_min_em(symbol: str = "000001",
                               start_time: str = "09:00:00",
                               end_time: str = "15:50:00",
                               ) -> pd.DataFrame:
    """
    东方财富网-行情首页-沪深京 A 股-每日分时行情包含盘前数据
    http://quote.eastmoney.com/concept/sh603777.html?from=classic
    :param symbol: 股票代码
    :type symbol: str
    :param start_time: 开始时间
    :type start_time: str
    :param end_time: 结束时间
    :type end_time: str
    :return: 每日分时行情包含盘前数据
    :rtype: pandas.DataFrame
    """
    code_id_dict = code_id_map_em()
    url = "https://push2.eastmoney.com/api/qt/stock/trends2/get"
    params = {
        "fields1": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58",
        "ut": "fa5fd1943c7b386f172d6893dbfba10b",
        "ndays": "1",
        "iscr": "1",
        "iscca": "0",
        "secid": f"{code_id_dict[symbol]}.{symbol}",
        "_": "1623766962675",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]["trends"]])
    temp_df.columns = [
        "时间",
        "开盘",
        "收盘",
        "最高",
        "最低",
        "成交量",
        "成交额",
        "最新价",
    ]
    temp_df.index = pd.to_datetime(temp_df["时间"])
    date_format = temp_df.index[0].date().isoformat()
    temp_df = temp_df[date_format + ' ' + start_time:date_format + ' ' + end_time]
    temp_df.reset_index(drop=True, inplace=True)
    temp_df["开盘"] = pd.to_numeric(temp_df["开盘"])
    temp_df["收盘"] = pd.to_numeric(temp_df["收盘"])
    temp_df["最高"] = pd.to_numeric(temp_df["最高"])
    temp_df["最低"] = pd.to_numeric(temp_df["最低"])
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"])
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"])
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"])
    temp_df['时间'] = pd.to_datetime(temp_df['时间']).astype(str)
    
    temp_df.columns = [
        "time",
        "open",
        "close",
        "high",
        "low",
        "vol",
        "amount",
        "last"
        ]

    return temp_df


if __name__ == "__main__":

    stock_zh_a_spot_em_df = stock_zh_a_spot_em()
    print(stock_zh_a_spot_em_df)

    stock_zh_a_hist_df = stock_zh_a_hist_em(
        symbol="600519", period='daily', start_date="20210101", end_date="20221231", adjust="hfq")
    print(stock_zh_a_hist_df)

    stock_zh_a_hist_min_em_df = stock_zh_a_hist_min_em(symbol="833454")
    print(stock_zh_a_hist_min_em_df)

    stock_zh_a_hist_pre_min_em_df = stock_zh_a_hist_pre_min_em(symbol="833454")
    print(stock_zh_a_hist_pre_min_em_df)

    stock_zh_a_hist_min_em_df = stock_zh_a_hist_min_em(symbol="000001", period='60', adjust='hfq', start_date="2022-04-15 09:32:00", end_date="2022-04-15 14:40:00")
    print(stock_zh_a_hist_min_em_df)

    stock_zh_a_hist_df = stock_zh_a_hist_em(symbol="833454", period="daily", start_date="20170301", end_date='20211115', adjust="hfq")
    print(stock_zh_a_hist_df)

   


from rrdata.common import save_df_to_pgsql

import pandas as pd
file = "/home/romep/rrdata/rrdata/rrdatad/index/stock_swindex_class_all.csv"

path_file = u"/home/romep/rrdata/rrdata/rrdatad/index/swsindex/swsindex_all_stock_class_new.xlsx"

df = pd.read_excel(path_file)

save_df_to_pgsql(df, "swl_stock_class_all_2021_cn")
print(df)

print(df.columns)







from sqlalchemy import Column, Integer, String, ForeignKey
from sqlalchemy import create_engine
from sqlalchemy.orm import declarative_base
from rrdata.utils.rqSetting import setting
import psycopg2

from rrdata.utils.rqLogs import rq_util_log_info

passwd_postgres = setting["PGSQL_PASSWORD"]
host_ip = setting["IP_DATABASE_ALIYUN"]
db_name = setting["DATABASE"]
table_name = "cities"

DATABASE_URL = f"postgresql+psycopg2://postgres:{passwd_postgres}@{host_ip}:5432/{db_name}"

engine = create_engine(DATABASE_URL, echo=True)
rq_util_log_info(engine)
# declarative base class
Base = declarative_base()

# an example mapping using the base
class City(Base):
    __tablename__ = table_name

    id = Column(Integer, autoincrement=True, primary_key=True, index=True)
    name = Column(String, unique=True)
    population = Column(Integer)
   
def add_city(session: engine, name: str, population: int):
    new_city = City(name=name, population=population)
    rq_util_log_info(f" add new_city: name: {name}, population: {population}")
    session.add(new_city)
    return new_city

if __name__ == '__main__':
    add_city(engine,"shenzhen",1700)


from dataclasses import replace
from functools import reduce
from heapq import merge
from numpy import NaN
import pandas as pd 
from rrdata.common import save_df_to_pgsql, engine
import rrdata.rrdatad.index  as index
from rrdata.utils.rqLogs import rq_util_log_info
from rrdata.utils.rqParameter import SWL_LEVEL


def save_index_sw_class_pgsql(table_name='swl_list'):
    data = index.sw_index_class_all()
    #print(data)
    save_df_to_pgsql(data, table_name, engine)
    #print(read_data_from_pg("swl_list"))


def save_index_sw_spot(table_name="swl_spot", level =""):
    """ save swl index L1, L2 realtime price
    """
    for l in ["L1", "L2"]:
        data = index.sw_index_spot(l)
        save_df_to_pgsql(data, f"{table_name}_{l}", engine)


def  save_index_sw_cons_one_level(table_name='swl_cons',level=""):
    """
    for leve in SWL_LEVEL().LEVEL get all cons
    table_name = f"table_name_{level}"
    important: 
    drop_duplicates(subset=["stock_code"], keep='last', inplace=True)
    """
    dfs =[]
    for symbol in index.sw_index_class(level).index_code.values:
        rq_util_log_info(symbol)
        try:
            cons = index.sw_index_legulegu_cons(symbol)
            cons.replace("-", NaN, inplace=True)
            cons.dropna(how='all', axis=1, inplace=True)
            #cons.sort_values(by=['stock_code', 'in_time'], ascending=[True, True], inplace=True)
            #cons.drop_duplicates(subset=['stock_code'], keep='first', inplace=True)
            rq_util_log_info(cons)
            dfs.append(cons)
            #rq_util_log_info(dfs)
        except:
            pass
    df = pd.concat(dfs)
    df.sort_values(by=['stock_code', 'in_time'], ascending=[True, True], inplace=True)
    df.drop_duplicates(subset=['stock_code'], keep='last', inplace=True)
    rq_util_log_info(df)
    save_df_to_pgsql(df, f'{table_name}_{level}', engine)


def save_swl_cons_all_level(table_name='swl_cons_all'):
    from rrdata.common import read_df_from_table
    from rrdata.utils.rqParameter import SWL_LEVEL
    def read_sw_index_cons_L(level=""):
        return read_df_from_table(f"swl_cons_{level}", con=engine)
    df_list =[]   
    for L in SWL_LEVEL().LEVEL:
        df_list.append(f"df{L[-1]}")
        df_list[-1] = read_sw_index_cons_L(level=L)
        df_list[-1].replace("-", NaN, inplace=True)
        #rq_util_log_info(df)
        df_list[-1].dropna(how='all', axis=1, inplace=True)
        df_list[-1].drop_duplicates(subset=["stock_code"], keep='last', inplace=True)
        df_list[-1].drop(['no', 'in_time'], axis=1, inplace=True)
        rq_util_log_info(df_list[-1])
       
    df_m = pd.merge(df_list[0], df_list[1],  how='outer')
    df = pd.merge(df_m, df_list[2], how='outer')
    rq_util_log_info(df)
    save_df_to_pgsql(df, 'swl_cons_all', con=engine)


if  __name__ == '__main__':
    save_index_sw_class_pgsql()
    save_index_sw_spot()
    for l in SWL_LEVEL().LEVEL:
        save_index_sw_cons_one_level('swl_cons', l)
        
    save_swl_cons_all_level()
    







# -*- coding: utf-8 -*-
from operator import index
import time
import pandas as pd
import psycopg2
from sqlalchemy import create_engine, engine_from_config
import logging

from rrdata.utils.rqSingleton import  ConSqlDb
from rrdata.utils.rqLogs import (rq_util_log_debug, rq_util_log_expection,
                                             rq_util_log_info)
from rrdata.utils.rqDate_trade import (rq_util_get_trade_range, rq_util_get_last_tradedate, 
                        rq_util_get_pre_trade_date, setting)
from rrdata.utils.rqSetting import DATABASE, POSTGRESQL_URI

logging.basicConfig(level=logging.INFO, format=' %(asctime)s- %(levelname)s-%(message)s')
password_pgsql = setting['PGSQL_PASSWORD']
pgsql_uri = setting['POSTGRESQL_URI']
host_db_ip = setting['IP_DATABASE_ALIYUN']
port_psql = 5432

class PgsqlOperation(object):
    """
    user/password: postgres
    host_default='127.0.0.1'
    host_database_server='39.108.68.163'
    port_default=5432
    execute:
    CREATE DATABASE db_name;
    DROP DATABASE [ IF EXISTS ] db_name;
    CREATE TABLE table_name(
   column1 datatype,
   column2 datatype,
   .....
   columnN datatype,
   PRIMARY KEY( 一个或多个列 )
);
    DROP TABLE table_name;
    PostgreSQL 模式（SCHEMA）:允许多个用户使用一个数据库并且不会互相干扰。
    CREATE SCHEMA myschema.mytable (
...
);
    create table myschema.table_name()
    DROP SCHEMA myschema;
    https://www.runoob.com/postgresql/postgresql-insert-into.html

use sqlalchemy:

from sqlalchemy import create_engine
engine = create_engine("postgresql+psycopg2://scott:tiger@host/dbname")
    
https://docs.sqlalchemy.org/en/14/dialects/postgresql.html
To set using per-connection execution options:

with engine.connect() as conn:
    conn = conn.execution_options(
        isolation_level="REPEATABLE READ"
    )
    with conn.begin():
        # ... work with transaction
    """

def create_psqlDB(db_name=None):
        try:
            conn = psycopg2.connect(user='postgres',
                                    database='postgres',
                                    password=password_pgsql,
                                    host=host_db_ip, 
                                    port=port_psql)
            #print(conn)
            psql = f'''CREATE DATABASE {db_name}'''
            #OWNER {self.user};
            #        GRANT ALL PRIVILEGES ON DATABASE {db_name}  to {self.user};
            #        '''
            conn.autocommit = True
            cursor = conn.cursor()
            cursor.execute(psql)
            #conn.commit()
            print(f'database {db_name} created successfully use by user postgres')
            #cursor.close()
            conn.close()
        except (Exception, psycopg2.DatabaseError) as error:
            print('Error while create PostgresSQL database', error)

class PgsqlClass(ConSqlDb):
    """ postgresql operation:
        first connect database uri from config.json;
        execute database 
    """  

    def __init__(self):
        self.user =  'postgres'
        self.password = password_pgsql
        self.host = host_db_ip
        self.port = 5432
        # self.engine = create_engine(f'postgresql+psycopg2://{self.user}:{self.password}@{self.host}:{self.port}/{self.database}')


    def client_pgsql(self, db_name):
        client = psycopg2.connect(user='postgres',
                                    database=db_name,
                                    password=self.password,
                                    host=self.host, 
                                    port=self.port)
        #client = create_engine(f'postgres+psycopg2://{self.user}:{self.password}@{self.host}:{self.port}/{db_name}')
        return client


    def create_table(self, db_name, table_name, tsql):
        try:
            conn = self.client_pgsql(db_name=db_name)
            cursor = conn.cursor()
            conn.autocommit = True
            cursor.execute(f"DROP TABLE if EXISTS {table_name}")
            cursor.execute(tsql)
            print(f'{table_name} created  successfully on database {db_name}')
            conn.close()
        except Exception as e:
            print(e)


    def insert_to_psql(self, df,db_name, table_name, if_exists='append'):
        try:
            engine = self.client_pgsql(db_name)
            df.to_sql(name=table_name, con=engine, index=False, if_exists=if_exists)
            print(f'write to database:{db_name} table:{table_name} ok !')
        except Exception as e:
            print(e)


    def save_df_to_pgsql(self, df, table_name, db_name, if_exists='replace'):
        engine = self.client_pgsql(db_name)
        df.to_sql(table_name,client=engine,index=False,if_exists=if_exists)


    def read_psql(self, db_name, table_name):
        engine = self.client_pgsql(db_name=db_name)
        return pd.read_sql_table(table_name,engine)



def client_pgsql(database='rrdata'):
    try:
        client = create_engine(f'postgresql+psycopg2://postgres:{password_pgsql}@{host_db_ip}:{port_psql}/{database}')
        return client
    except Exception as e:
        print(e)     


def save_data_to_postgresql(data,table_name,db_name,if_exists='replace'):
    client = client_pgsql(database=db_name)
    data.to_sql(table_name=table_name,client=client,index=False,if_exists=if_exists)
    

def load_data_from_postgresql(mes='',client=client_pgsql("rrdata")):
    res=pd.read_sql(mes,client)
    return res


def read_data_from_pg(table_name='', client=client_pgsql("rrdata")):
    ''' load all data of table_name
    '''
    res = pd.read_sql_table(table_name, client)
    return res


def read_table_from_pg(table_name='',period=1, client=None):
    '''only select trade_date, can't select columns
        default out lasttradedate data
        period -->int >= 1
    '''
    try: 
        start_date = rq_util_get_pre_trade_date(rq_util_get_last_tradedate(), period-1)
        logging.info(start_date)
        sql = ''.join(['SELECT * FROM ', f'{table_name}', f" WHERE trade_date >= date_trunc('day', timestamp '"+start_date+"') order by trade_date ASC;"])
        logging.info(sql)
        res = pd.read_sql(sql,client)
        logging.info(f'\n {res}')
        return res
    except Exception as e:
        logging.info(e)


def read_sql_from_pg(start_date=None,data=None,table_name=None,client=client_pgsql()):
    '''can select columns and trade_date
    '''
    sql = ''.join(['SELECT ', data, ' FROM ', f'{table_name}', f" WHERE trade_date >= date_trunc('day', timestamp '"+start_date+"') order by trade_date ASC;"])
    #sql= 'select ' + data + ' from ' +table_name+" where trade_date >= date_trunc('day',timestamp '"+start_date+"') order by trade_date ASC;"
    try:
        t=time.time()
        res=pd.read_sql(sql,client)
        t1=time.time()
        tt = t1-t
        logging.info(f'read  data from {table_name} ,take {tt}')
        return res
    except Exception as e:
        logging.error(e)


def read_unique_data_from_pg(cols='*',table_name='',period=1, client=client_pgsql()):
    ''' select trade_date and columns( * is all), data GROUP BY trade_date and _ts_code --> duplicates 
    '''
    try: 
        start_date = rq_util_get_pre_trade_date(rq_util_get_last_tradedate(), period-1)
        logging.info(start_date)
        sql = f"""
        SELECT {cols} 
        FROM  {table_name}
        GROUP BY trade_date, ts_code 
        """        
        logging.info(sql)
        res = pd.read_sql(sql,client)
        logging.info(f'\n {res}')
        return res
    except Exception as e:
        logging.info(e)


if __name__ == '__main__':
    import os
    #create_psqlDB("rralpha")

    psql = PgsqlClass()
    #psql.create_table('rrdata','swl_list', '')
   
    
    """
    path = '/home/rome/rrdata/rrdata/sql'
    file_n = 'stock_day.sql'
    file_path = os.path.join(path, file_n)
    print(file_path)
    with open(file_path, 'r') as f:
        tsql = f.read()
    print(tsql)
    psql.create_table('rrdata','stock_day',tsql)
    """
    #psql2 = PgsqlClass()
    #print(id(psql), id(psql2))
    #psql.client_pgsql(db_name='rrdata')
    
    #print(read_data_from_pg('swl_list'))

    #print(read_table_from_pg(table_name='swl_day',client=client_pgsql('rrdata')))
    



"""
from .index import (sw_index_spot,
                    sw_index_daily,
                    sw_index_daily_indicator,
                    sw_index_class,
                    sw_index_cons,
                    sw_index_class_all
                    )
"""
from .index import *
from .stock import *


import json
import os , platform

from rrdata.utils.rqLogs import rq_util_log_info
from rrdata.utils.rqSingleton import Singleton, Singleton_wraps

config_file_dir = ".rrsdk"

# diffrence OS path
path = os.path.expanduser('~')
#rq_util_log_info(path)
rq_path = f'{path}{os.sep}{config_file_dir}' #.format(path, os.sep, '.rrsdk')
#rq_util_log_info(f"{platform.system()}, {path}, {rq_path}")


def get_path_file_name(path_name, file_name):
    # diffrence OS path
    if platform.system() == 'Linux':
        path_name_chg = ''.join([rq_path,'/' ,path_name, '/', file_name])
    if platform.system() == 'Windows':
        path_name_chg =  ''.join([rq_path,'\\', path_name, '\\',file_name])
    #rq_util_log_info(path_name_chg)
    return path_name_chg

path_setting = get_path_file_name('setting','config.json')
rq_util_log_info(path_setting)
path_setting_ini =  get_path_file_name('setting','config.ini')
rq_util_log_info(path_setting_ini)

@Singleton_wraps
class Setting(object):
    def __init__(self, path_config=path_setting):
        self.path_config = path_config
        
    def setting(self):
        config = open(self.path_config)
        return json.load(config)

setting = Setting().setting()
#print(setting)


if __name__ == '__main__':
    #get_path_file_name("setting","config.ini")
    s = Setting()
    #print(id(s), id(Setting()))
    #print(s.setting()['TSPRO_TOKEN'])
    rq_util_log_info(s.setting()['IP_DATABASE_ALIYUN'])

    


#!/usr/bin/env python
import time
import os
import datetime
from venv import create
import asyncpg
import logging
import sqlite3
from typing import Dict


log = logging.getLogger("RRSDK")
logging.basicConfig(level=logging.INFO)


async def check_database(pool: asyncpg.pool.Pool):
    """ check db"""
    try:
        async with pool.acquire() as con:
            await create_database(con)
    except asyncpg.InsufficientPrivilegeError as e:
        log.error(f"error: {e}")
        return False
    return True


tables = [
    """
    CREATE TABLE "stock_lists" (
        id serial NOT NULL,
        stock_name text NOT NULL,

        created timestamptz DEFAULT now(),
        PRIMARY KEY (id),
        UNIQUE(STOCK_NAME)
    )
    """,
    """
    """
]

async def import_legacy_db(pool: asyncpg.pool.Pool, path):
    if not os.path.isfile(path):
        log.error("base file is not exist")
        return
    legacy_conn = sqlite3.connect(path)
    c = legacy_conn.cursor()
    log.info("")

async def create_database(con:asyncpg.connection.Connection):
    """ cteate rrdata's tables and functions"""
    log.info("create tables test")
    for create_query in tables:
        await con.execute(create_query)
    





#!/usr/bin/python
""" see bigquant doc
    DataSource("basic_info_IndustrySw").read(instruments=stock_lists)
    out: DataFrame
"""

from rrdata.common import read_df_from_table, save_df_to_pgsql
from rrdata.common.engine_pgsql import engine


class RrdataD(object):
    """ see:
        bigquant DataSource("trade_days").read() 
        also add  can change:
        source database/sql_driver
    """
    def __init__(self,table_name, 
               datasource=engine(driver="", db_name="rrdata"),
                ):
        """ driver: ""-default - poasgresql/ psycopg2   db_name  rrdata/rrshare/rrfactor"""
        self.table_name = table_name
        self.engine = datasource

    def __repr__(self) -> str:  # name 
        return f"<{self.__class__.__name__} table_name={self.table_name}>"

    def read(self):  # read artubuite
        return  read_df_from_table(self.table_name, self.engine)

    #def save(self, data):
    #    return save_df_to_pgsql(data, self.table_name)


if  __name__ == "__main__":
    print(RrdataD("swl_list").__repr__())
    RrdataD('stock_list').read()
    #RrdataD('trade_days').read()
    






#!/usr/bin/python
import  psycopg2

from rrdata.utils.config_postgresql import get_config_ini
from rrdata.utils.rqLogs import rq_util_log_info, rq_util_log_expection

db_name_rrdata = "pgsql_rrdata"

class PgsqlClass(object):

    def __init__(self,
            db_name=db_name_rrdata
            ):
        self.db_name=db_name
        self.params =get_config_ini(section=db_name)


    def create_tables(self, sql):
        """ create tables in the PostgreSQL database"""
        conn = None
        try:
            # read the connection parameters
            #params = get_config_ini(section=self.db_name)
            # connect to the PostgreSQL server
            with psycopg2.connect(**self.params) as conn:
                rq_util_log_info(conn)
                cur = conn.cursor()
                # create table one by one
                cur.execute(sql)
                # close communication with the PostgreSQL database server
                cur.close()
                # commit the changes
                conn.commit()
                rq_util_log_info(f"create table use cli: {sql}")
        except (Exception, psycopg2.DatabaseError) as error:
            rq_util_log_expection(error)


    def insert_one(self,sql_one,table_name):
        pass


    def insert_lists(self, sql_lists,table_name):
        pass


    def update(self):
        pass


    def fetchone(self):
        pass


    def fetchall(self, table_name='cities'):
        conn = None
        try:
            with psycopg2.connect(**self.params) as conn:
                cur = conn.cousor()
                cur.execute("""
                
                """)
                cur.close()
                cur.commit()
        except (Exception, psycopg2.DatabaseError) as error:
            rq_util_log_expection(error)

    

if  __name__ == '__main__':
    psql = PgsqlClass()
    sql="""
        DROP TABLE IF EXISTS cities;
        CREATE TABLE IF NOT EXISTS cities (
            id SERIAL PRIMARY KEY,
            name VARCHAR(255) NOT NULL,
            population INT NOT NULL
        )
        """
    psql.create_tables(sql)



from .rrdatad import index
from .rrdatad import stock


"""
from .rrdatad import (sw_index_spot, 
                      sw_index_daily,
                      sw_index_daily_indicator,
                      sw_index_class,
                      sw_index_class_all,
                      sw_index_cons,
                      )
"""


#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2022/2/23 21:22
Desc: 东方财富网-数据中心-年报季报-分红送配
http://data.eastmoney.com/yjfp/
"""
import pandas as pd
import requests
from tqdm import tqdm


def stock_fhps_em(date: str = "20210630") -> pd.DataFrame:
    """
    东方财富网-数据中心-年报季报-分红送配
    http://data.eastmoney.com/yjfp/
    :param date: 分红送配报告期
    :type date: str
    :return: 分红送配
    :rtype: pandas.DataFrame
    """
    url = 'https://datacenter-web.eastmoney.com/api/data/v1/get'
    params = {
        'sortColumns': 'PLAN_NOTICE_DATE',
        'sortTypes': '-1',
        'pageSize': '500',
        'pageNumber': '1',
        'reportName': 'RPT_SHAREBONUS_DET',
        'columns': 'ALL',
        'quoteColumns':'',
        'js': '{"data":(x),"pages":(tp)}',
        'source': 'WEB',
        'client': 'WEB',
        'filter': f"""(REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')"""
    }

    r = requests.get(url, params=params)
    data_json = r.json()
    total_pages = int(data_json['result']['pages'])
    big_df = pd.DataFrame()
    for page in tqdm(range(1, total_pages + 1), leave=False):
        params.update({'pageNumber': page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json['result']['data'])
        big_df = pd.concat([big_df, temp_df], ignore_index=True)

    big_df.columns = [
        '_',
        '名称',
        '_',
        '_',
        '代码',
        '送转股份-送转总比例',
        '送转股份-送转比例',
        '送转股份-转股比例',
        '现金分红-现金分红比例',
        '预案公告日',
        '股权登记日',
        '除权除息日',
        '_',
        '方案进度',
        '_',
        '最新公告日期',
        '_',
        '_',
        '_',
        '每股收益',
        '每股净资产',
        '每股公积金',
        '每股未分配利润',
        '净利润同比增长',
        '总股本',
        '_',
        '现金分红-股息率',
        '-',
        '-',
        '-',
    ]
    big_df = big_df[[
        '代码',
        '名称',
        '送转股份-送转总比例',
        '送转股份-送转比例',
        '送转股份-转股比例',
        '现金分红-现金分红比例',
        '现金分红-股息率',
        '每股收益',
        '每股净资产',
        '每股公积金',
        '每股未分配利润',
        '净利润同比增长',
        '总股本',
        '预案公告日',
        '股权登记日',
        '除权除息日',
        '方案进度',
        '最新公告日期',
    ]]
    big_df['送转股份-送转总比例'] = pd.to_numeric(big_df['送转股份-送转总比例'])
    big_df['送转股份-送转比例'] = pd.to_numeric(big_df['送转股份-送转比例'])
    big_df['送转股份-转股比例'] = pd.to_numeric(big_df['送转股份-转股比例'])
    big_df['现金分红-现金分红比例'] = pd.to_numeric(big_df['现金分红-现金分红比例'])
    big_df['现金分红-股息率'] = pd.to_numeric(big_df['现金分红-股息率'])
    big_df['每股收益'] = pd.to_numeric(big_df['每股收益'])
    big_df['每股净资产'] = pd.to_numeric(big_df['每股净资产'])
    big_df['每股公积金'] = pd.to_numeric(big_df['每股公积金'])
    big_df['每股未分配利润'] = pd.to_numeric(big_df['每股未分配利润'])
    big_df['净利润同比增长'] = pd.to_numeric(big_df['净利润同比增长'])
    big_df['总股本'] = pd.to_numeric(big_df['总股本'])

    big_df['预案公告日'] = pd.to_datetime(big_df['预案公告日'], errors="coerce").dt.date
    big_df['股权登记日'] = pd.to_datetime(big_df['股权登记日'], errors="coerce").dt.date
    big_df['除权除息日'] = pd.to_datetime(big_df['除权除息日'], errors="coerce").dt.date
    big_df['最新公告日期'] = pd.to_datetime(big_df['最新公告日期'], errors="coerce").dt.date
    return big_df


if __name__ == '__main__':
    stock_fhps_em_df = stock_fhps_em(date="20211231")
    print(stock_fhps_em_df)



#!/usr/bin/env bash

import pandas as  pd
import requests
from bs4 import BeautifulSoup


def sw_index_cons(symbol: str = "801011") -> pd.DataFrame:
    """
    申万指数成份信息-包括一级和二级行业都可以查询
    http://www.swsindex.com/idx0210.aspx?swindexcode=801010
    :param symbol: 指数代码
    :type symbol: str
    :return: 申万指数成份信息
    :rtype: pandas.DataFrame
    """
    url = f"http://www.swsindex.com/downfile.aspx?code={symbol}"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "html5lib")
    data = []
    table = soup.findAll("table")[0]
    rows = table.findAll("tr")
    for row in rows:
        cols = row.findAll("td")
        if len(cols) >= 4:
            stock_code = cols[0].text
            stock_name = cols[1].text
            weight = cols[2].text
            start_date = cols[3].text

            data.append(
                {
                    "stock_code": stock_code,
                    "stock_name": stock_name,
                    "start_date": start_date,
                    "weight": weight,
                }
            )
    temp_df = pd.DataFrame(data)
    temp_df["start_date"] = pd.to_datetime(temp_df["start_date"]).dt.date
    temp_df["weight"] = pd.to_numeric(temp_df["weight"])
    return temp_df


def sw_index_legulegu_cons(symbol: str = "851921.SI") -> pd.DataFrame:
    """
    乐咕乐股-申万一，二，三级-行业成份
    https://legulegu.com/stockdata/index-composition?industryCode=851921.SI
    :param symbol: 三级行业的行业代码
    :type symbol: str
    :return: 行业成份
    :rtype: pandas.DataFrame
    """
    url = f"https://legulegu.com/stockdata/index-composition?industryCode={symbol}"
    temp_df = pd.read_html(url)[0]
    temp_df.columns = [
        "no",
        "stock_code",
        "display_name",
        "in_time",
        "sw_L1",
        "sw_L2",
        "sw_L3",
        "close",
        "pe",
        "pe_ttm",
        "pb",
        "dy",
        "mv",
    ]
    temp_df["close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    temp_df["pe"] = pd.to_numeric(temp_df["pe"], errors="coerce")
    temp_df["pe_ttm"] = pd.to_numeric(temp_df["pe_ttm"], errors="coerce")
    temp_df["pb"] = pd.to_numeric(temp_df["pb"], errors="coerce")
    temp_df["dy"] = pd.to_numeric(temp_df["dy"].str.strip("%"), errors="coerce")
    temp_df["mv"] = pd.to_numeric(temp_df["mv"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    try:
        print(sw_index_cons(symbol="801011"))
        print(sw_index_cons(symbol="801016"))
        print(sw_index_legulegu_cons("801017.SI"))
    except:
        pass








#!/usr/bin/python
import pandas
import tempfile
from sqlalchemy import create_engine

#from rrdata.utils.rqSql import rq_util_sql_postgres_setting
from rrdata.utils.rqSetting import setting
from rrdata.utils.client_postgsql import get_config_ini
from rrdata.utils.rqLogs import rq_util_log_info


engine = get_config_ini(section='pgsql_rrdata')
print(engine)

database_uri = setting['POSTGRESQL']
rq_util_log_info(database_uri)
engine = create_engine(database_uri)
rq_util_log_info(engine)


def read_sql_tmpfile(query, db_engine):
    """ db_engine must create_engine("postgresql://host/dbname")
    """
    with tempfile.TemporaryFile() as tmpfile:
        copy_sql = "COPY ({query}) TO STDOUT WITH CSV {head}".format(
           query=query, head="HEADER"
        )
        conn = db_engine.raw_connection()
        cur = conn.cursor()
        cur.copy_expert(copy_sql, tmpfile)
        tmpfile.seek(0)
        df = pandas.read_csv(tmpfile)
        return df

if  __name__ == '__main__':
    query = """ SELECT * FROM swl_list
  
    """
    df = read_sql_tmpfile(query,engine)
    print(df)



from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy.ext.asyncio import create_async_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy import Column
from sqlalchemy import String
from sqlalchemy import Integer
from sqlalchemy import select

from rrdata.utils.rqSetting import setting
from rrdata.utils.rqLogs import rq_util_log_info

passwd_postgres = setting["PGSQL_PASSWORD"]
host_ip = setting["IP_DATABASE_ALIYUN"]
db_name = setting["DATABASE"]
table_name = "cities"

DATABASE_URL = f"postgresql+asyncpg://postgres:{passwd_postgres}@{host_ip}:5432/{db_name}"

engine = create_async_engine(DATABASE_URL, echo=True)
rq_util_log_info(engine)
#Base class
Base = declarative_base()
async_session = sessionmaker(
    engine, class_=AsyncSession, expire_on_commit=False
)
rq_util_log_info(async_session)


class City(Base):
    __tablename__ = table_name

    id = Column(Integer, autoincrement=True, primary_key=True, index=True)
    name = Column(String, unique=True)
    population = Column(Integer)


async def init_models():
    async with engine.begin() as conn:
        await conn.run_sync(Base.metadata.drop_all)
        await conn.run_sync(Base.metadata.create_all)


# Dependency
async def get_session() -> AsyncSession:
    async with async_session() as session:
        yield session


async def get_biggest_cities(session: AsyncSession) -> list[City]:
    result = await session.execute(select(City).order_by(City.population.desc()).limit(20))
    return result.scalars().all()


def add_city(session: AsyncSession, name: str, population: int):
    new_city = City(name=name, population=population)
    rq_util_log_info(f" add new_city: name: {name}, population: {population}")
    session.add(new_city)
    return new_city


if  __name__ == '':
    #get_biggest_cities()
    add_city(get_session(),"shenzhen",1700)


from rrdata.rrdatad.stock.fetch_stock_list import fetch_stock_list_tusharepro
from rrdata.common import save_df_to_pgsql
from rrdata.rrdatad.stock.stock_zh_a_hist_em import stock_zh_a_spot_em

save_df_to_pgsql(fetch_stock_list_tusharepro(), 'stock_list')

#stock_spot = stock_zh_a_spot_em()

save_df_to_pgsql(stock_zh_a_spot_em(), 'stock_spot')





from psycopg2 import connect
from sqlalchemy import create_engine
import pandas as pd
import tempfile

import rrdata.rrdatad.index as index
from rrdata.utils.rqLogs import rq_util_log_info, rq_util_log_expection
from rrdata.utils.rqSetting import setting
from rrdata.common import engine, engine_async


def read_df_from_table(table_name, con=engine()):
    """read whole table all dataframe from eginee() """
    try:
        df = pd.read_sql(table_name, con)
        rq_util_log_info(f"Read dataframe: {len(df)} rows from Table:<{table_name}> from {con} \n{df}")
        return df
    except Exception as e:
        rq_util_log_expection(e)


def read_data_from_table(sql_query, con=engine()):
    """sql_query= f'SELECT * FROM {table_name} WHERE trade_date BEETWIN start_date, end_date ' """
    try:
        df = pd.read_sql_query(sql_query, con)
        rq_util_log_info(f"Read dataframe: {len(df)} rows from Table from {con} \n{df}")
        return df
    except Exception as e:
        rq_util_log_expection(e)


def read_large_df_from_table(sql_query, con=engine()):
    
    """    pandas has a built-in chunksize parameter 
    that you can use to control this sort of thing
    """

    dfs = []
    for chunk in pd.read_sql_query(sql_query, con, chunksize=5000):
        dfs.append(chunk)
    df = pd.concat(dfs)
    rq_util_log_info(df())
    return df 


def read_sql_tmpfile(query, con=engine()):
    with tempfile.TemporaryFile() as tmpfile:
        copy_sql = "COPY ({query}) TO STDOUT WITH CSV {head}".format(
           query=query, head="HEADER"
        )
        connect = con.raw_connection()
        cur = connect.cursor()
        cur.copy_expert(copy_sql, tmpfile)
        tmpfile.seek(0)
        df = pd.read_csv(tmpfile)
        rq_util_log_info(df)
        return df


if __name__ == '__main__':
    #for t in ['swl_cons_L1','swl_cons_L2','swl_cons_L3']:
    #    read_df_from_table(t)

    #sql_query = 'SELECT * FROM swl_list'
    #read_large_df_from_table(sql_query)
    #read_sql_tmpfile(sql_query, engine)
    #read_df_from_table('stock_spot', con=engine(db_name="rrdata"))
    table_name = "stock_day_test"
    start_date = "2022-05-11"
    end_date = "2022-05-17"
    sql1 = f"""
        SELECT ts_code, trade_date, close, chg_pct 
        FROM {table_name}
        WHERE trade_date BETWEEN  '{start_date}' AND '{end_date}';
    """
    read_data_from_table(sql1)



import asyncio
import asyncpg
import json
import datetime

from rrdata.common.engine_pgsql import database_dns, engine


async def main(db_name='rrdata'):
    conn = await asyncpg.connect(f"{database_dns}/{db_name}")

    try:
        await conn.set_type_codec(
            'json',
            encoder=json.dumps,
            decoder=json.loads,
            schema='pg_catalog'
        )

        data = {'foo': 'bar', 'spam': 1}
        res = await conn.fetchval('SELECT $1::json', data)

    finally:
        await conn.close()

async def main2(db_name='rrdata'):
    # Establish a connection to an existing database named "test"
    # as a "postgres" user.
    conn = await asyncpg.connect(f"{database_dns}/{db_name}")
    # Execute a statement to create a new table.
    await conn.execute('''
        CREATE TABLE IF NOT EXISTS users (
            id serial PRIMARY KEY,
            name text,
            dob date
        )
    ''')

    # Insert a record into the created table.
    await conn.execute('''
        INSERT INTO users(name, dob) VALUES($1, $2)
    ''', 'Bob', datetime.date(1984, 3, 1))

    # Select a row from the table.
    row = await conn.fetchrow(
        'SELECT * FROM users WHERE name = $1', 'Bob')
    # *row* now contains
    print(row)
    #asyncpg.Record(id=1, name='Bob', dob=datetime.date(1984, 3, 1))

    # Close the connection.
    await conn.close()

if  __name__ == "__main__":
    asyncio.get_event_loop().run_until_complete(main())
    from rrdata.common import read_df_from_table
    #print(read_df_from_table('swl_list',engine()))
    #asyncio.get_event_loop().run_until_complete(main2())
    #print(read_df_from_table("users"))


from ast import If
import pandas as pd

from rrdata.utils.rqTusharepro import pro


def fetch_stock_list_tusharepro():    
    stock_list_l= pro.stock_basic(exchange_id='', is_hs='',list_status='L' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')  
    stock_list_D= pro.stock_basic(exchange_id='', is_hs='',list_status='D' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')  
    stock_list_P= pro.stock_basic(exchange_id='', is_hs='',list_status='P' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')          
    stock_list=pd.concat([stock_list_l,stock_list_D],axis=0)
    stock_list=pd.concat([stock_list,stock_list_P],axis=0)
    return stock_list


if __name__ == '__main__':
    print(fetch_stock_list_tusharepro())


import pandas as pd
import asyncio
from rrdata.common import engine, engine_async


async def read_sql_async(stmt, con):
    loop = asyncio.get_event_loop()
    return await loop.run_in_executor(None, pd.read_sql, stmt, con)


async def engine_async(db_name='rrdata'):
    engine = create_async_engine(
        "postgresql+asyncpg://postgres:{passwd}@{host}:5432/{db_name}", echo=True,
    )

    async with engine.begin() as conn:
        await conn.run_sync(meta.drop_all)
        await conn.run_sync(meta.create_all)

        await conn.execute(
            t1.insert(), [{"name": "some name 1"}, {"name": "some name 2"}]
        )

    async with engine.connect() as conn:

        # select a Result, which will be delivered with buffered
        # results
        result = await conn.execute(select(t1).where(t1.c.name == "some name 1"))

        print(result.fetchall())

    # for AsyncEngine created in function scope, close and
    # clean-up pooled connections
    await engine.dispose()

asyncio.run(async_main())

if __name__ == '__main__':

    #loop = asyncio.get_event_loop()
    #df = loop.run_until_complete(main())
    #loop.close()
    """
    # Establish a connection to an existing database named "test"
    # as a "postgres" user.
    conn = await asyncpg.connect('postgresql://postgres@localhost/test')

    asyncio.get_event_loop().run_until_complete(main())
    """
    # tasks = await asyncio.gather(*tasks)
    #df = await read_sql_async('stock_list',engine('rrshare'))
    #print(df)


from rrdata.common import engine
from rrdata.common import read_df_from_table

def update_sql_table_from_df(df, con=engine(db_name='rrdata')): #TODO
    # Create the temporary table using pandas (can speed this up with method above if needed)
    df.to_sql('temporary_table', con=con, if_exists='replace')
    # Define a string that will run as a SQL command when executed
    # uses SQL UPDATE to update table_to_be_updated from temporary_table
    sql = """
        UPDATE table_to_be_updated AS f
        SET col1 = t.col1
        FROM temporary_table AS t
        WHERE f.id = t.id
    """
    with engine(db_name="rrdata").begin() as conn:     
        conn.execute(sql)


if __name__ == "__main__":
    data = read_df_from_table('stock_list')
    update_sql_table_from_df(data)


import pandas as pd

from rrdata.rrdatad.index import *

class SwIndex(object):

    def __init__(self) -> None:
        self.LEVEL=["L1","L2","L3"]
        

    def get_swl_class_cons(self,level="L1"):
        swl_cons = pd.DataFrame()
        for code in sw_index_class(level=level)['index'].values:
            print(code)
            try:
                swl_cons = pd.concat([swl_cons,sw_index_cons(symbol=code)])
            except Exception as e:
               pass
            print(swl_cons)
        return swl_cons


if __name__ == "__main__":
    swl = SwIndex()
    sw_cons = swl.get_swl_class_cons("L3")
    print(sw_cons)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rrdata',
 'rrdata.common',
 'rrdata.rrdatac',
 'rrdata.rrdatac.index',
 'rrdata.rrdatac.indicator_tech',
 'rrdata.rrdatac.utils',
 'rrdata.rrdatad',
 'rrdata.rrdatad.index',
 'rrdata.rrdatad.industry',
 'rrdata.rrdatad.stock',
 'rrdata.rrdatad.trade_calender',
 'rrdata.temp',
 'rrdata.utils']

package_data = \
{'': ['*'], 'rrdata.rrdatad.index': ['swsindex/*']}

setup_kwargs = {
    'name': 'rrdata',
    'version': '0.2.0',
    'description': '',
    'long_description': 'rrdata is python project for china stock analysis.\n\ndefault database id rrdata\n\nanalysis database is rralpha/rqfactor\n\n\nrrdatad is fetch all data form open source(web/api);\ninclude:\n(stock--sina/eastmoney/tusharepro;\nindex --- swsindex.com / legulegu.com / \nfund -- \n)\ncron  record  to database rrdata;\n\n\nrrdatac is api to get data from my database server or web(realtime/spot)\n\nsefult analysis alpha factor to database rralpha cronly.\n\n\n\nusage:\n\n1.rrdsk init;\n\na. run rqLocalize for mkdir ~/.rrsdk/setting for config\n\nb. cp config.json and config.ini to path(setting)\n\nc. config include: sql password / database-server-ip/name/port, sql-uri, tusharepro-token and so on.\n\nd. .rrsdk save to romepeng/.rrsdk.git (private) by git push;\n\n2. get rrdata --data(index/stock/fund/bond) by rrdatac(rrdata) api;\n\n   get quant alpha factor from database rqfactor or rralpha\n\n3. you can show your analysis results to web by streamlit / pyecharts / hightcharts  and so on;\n\n\n\n\n',
    'author': 'romepeng',
    'author_email': 'romepeng@outlook.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/romepeng/rrdata.git',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from .fetch_swl_index_spot import sw_index_spot
from .fetch_sw_index_daily import sw_index_daily, sw_index_daily_indicator
from .fetch_sw_index_class import sw_index_class, sw_index_class_all
from .fetch_sw_index_cons import sw_index_cons, sw_index_legulegu_cons




#!/usr/bin/env bash

import pandas as pd
import requests
from bs4 import BeautifulSoup


def sw_index_info(level="L2") -> pd.DataFrame:
    """
    乐咕乐股-申万三级-分类
    https://legulegu.com/stockdata/sw-industry-overview#level1
    :return: 分类
    :rtype: pandas.DataFrame
    """
    url = "https://legulegu.com/stockdata/sw-industry-overview"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    levelTtems=f"level{level[-1]}Items"
    code_raw = soup.find("div", attrs={"id": levelTtems}).find_all(
        "div", attrs={"class": "lg-industries-item-chinese-title"}
    )
    name_raw = soup.find("div", attrs={"id": levelTtems}).find_all(
        "div", attrs={"class": "lg-industries-item-number"}
    )
    value_raw = soup.find("div", attrs={"id": levelTtems}).find_all(
        "div", attrs={"class": "lg-sw-industries-item-value"}
    )
    code = [item.get_text() for item in code_raw]
    name = [item.get_text().split("(")[0] for item in name_raw]
    num = [item.get_text().split("(")[1].split(")")[0] for item in name_raw]
    num_1 = [
        item.find_all("span", attrs={"class": "value"})[0].get_text().strip()
        for item in value_raw
    ]
    num_2 = [
        item.find_all("span", attrs={"class": "value"})[1].get_text().strip()
        for item in value_raw
    ]
    num_3 = [
        item.find_all("span", attrs={"class": "value"})[2].get_text().strip()
        for item in value_raw
    ]
    num_4 = [
        item.find_all("span", attrs={"class": "value"})[3].get_text().strip()
        for item in value_raw
    ]
    temp_df = pd.DataFrame([code, name, num, num_1, num_2, num_3, num_4]).T
    temp_df.columns = [
        "index_code",
        "index_name",
        "cons_num",
        "pe",
        "pe_ttm",
        "pb",
        "dy",
    ]
    temp_df["cons_num"] = pd.to_numeric(temp_df["cons_num"])
    temp_df["pe"] = pd.to_numeric(temp_df["pe"])
    temp_df["pe_ttm"] = pd.to_numeric(temp_df["pe_ttm"])
    temp_df["pb"] = pd.to_numeric(temp_df["pb"])
    temp_df["dy"] = pd.to_numeric(temp_df["dy"])
    return temp_df


def sw_index_class(level="L1"):
    """
    swl L1, L2, L3 class
    乐咕乐股-申万一，二，三级-分类
    https://legulegu.com/stockdata/sw-industry-overview#level1
    """
    sw_level=f"level{level[-1]}"
    #print(sw_level)
    df_swl = pd.DataFrame()
    swl_spot= sw_index_info(sw_level)
    df_swl["index_code"] = swl_spot["index_code"]
    df_swl["index_symbol"] = df_swl["index_code"].map(lambda x: x[:6])
    df_swl["index_name"] = swl_spot["index_name"]
    df_swl["level"] = f"{level}"
    df_swl["name_level"] = swl_spot["index_name"] + "_" + df_swl["level"]
    print(f"get_swl_{level}_index")
    return df_swl


def sw_index_class_all():
    df = pd.DataFrame() 
    for level in ["L1", "L2", "L3"]:
        df = pd.concat([df,sw_index_class(level)])
    return df


if __name__ == '__main__':
    print(sw_index_info("L2"))
    print(sw_index_class_all())
  

